@@ -29,6 +29,10 @@ FuzzHelper::doChecks()
     Pl_TIFFPredictor p("decoder", &discard, Pl_TIFFPredictor::a_decode, 16, 1, 8);
     p.write(const_cast<unsigned char*>(data), size);
     p.finish();
+    // Exercise with strange values for some of the parameters.
+    Pl_TIFFPredictor p2("decoder", &discard, Pl_TIFFPredictor::a_decode, 16, 2, 5);
+    p2.write(const_cast<unsigned char*>(data), size);
+    p2.finish();
 }
 
 void
