@@ -3,7 +3,7 @@
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QUtil.hh>
 #include <algorithm>
-#include <string.h>
+#include <cstring>
 
 FileInputSource::FileInputSource() :
     close_file(false),
@@ -18,8 +18,7 @@ FileInputSource::FileInputSource(char const* filename) :
 {
 }
 
-FileInputSource::FileInputSource(
-    char const* description, FILE* filep, bool close_file) :
+FileInputSource::FileInputSource(char const* description, FILE* filep, bool close_file) :
     close_file(close_file),
     filename(description),
     file(filep)
@@ -28,8 +27,7 @@ FileInputSource::FileInputSource(
 
 FileInputSource::~FileInputSource()
 {
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in
-    // README-maintainer
+    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
     if (this->file && this->close_file) {
         fclose(this->file);
     }
@@ -69,8 +67,7 @@ FileInputSource::findAndSkipNextEOL()
             char* p = (p1 && p2) ? std::min(p1, p2) : p1 ? p1 : p2;
             if (p) {
                 result = cur_offset + (p - buf);
-                // We found \r or \n.  Keep reading until we get past
-                // \r and \n characters.
+                // We found \r or \n.  Keep reading until we get past \r and \n characters.
                 this->seek(result + 1, SEEK_SET);
                 char ch;
                 while (!done) {
@@ -104,8 +101,8 @@ FileInputSource::seek(qpdf_offset_t offset, int whence)
 {
     if (QUtil::seek(this->file, offset, whence) == -1) {
         QUtil::throw_system_error(
-            std::string("seek to ") + this->filename + ", offset " +
-            std::to_string(offset) + " (" + std::to_string(whence) + ")");
+            std::string("seek to ") + this->filename + ", offset " + std::to_string(offset) + " (" +
+            std::to_string(whence) + ")");
     }
 }
 
