@@ -5,10 +5,9 @@ Pl_Concatenate::Pl_Concatenate(char const* identifier, Pipeline* next) :
 {
 }
 
-Pl_Concatenate::~Pl_Concatenate()
+Pl_Concatenate::~Pl_Concatenate() // NOLINT (modernize-use-equals-default)
 {
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in
-    // README-maintainer
+    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
 }
 
 void
