@@ -7,10 +7,9 @@ Pl_Discard::Pl_Discard() :
 {
 }
 
-Pl_Discard::~Pl_Discard()
+Pl_Discard::~Pl_Discard() // NOLINT (modernize-use-equals-default)
 {
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in
-    // README-maintainer
+    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
 }
 
 void
