@@ -13,10 +13,9 @@ Pl_OStream::Pl_OStream(char const* identifier, std::ostream& os) :
 {
 }
 
-Pl_OStream::~Pl_OStream()
+Pl_OStream::~Pl_OStream() // NOLINT (modernize-use-equals-default)
 {
-    // Must be explicit and not inline -- see QPDF_DLL_CLASS in
-    // README-maintainer
+    // Must be explicit and not inline -- see QPDF_DLL_CLASS in README-maintainer
 }
 
 void
