@@ -19,17 +19,14 @@ void
 Pl_RC4::write(unsigned char const* data, size_t len)
 {
     if (this->outbuf == nullptr) {
-        throw std::logic_error(
-            this->identifier +
-            ": Pl_RC4: write() called after finish() called");
+        throw std::logic_error(this->identifier + ": Pl_RC4: write() called after finish() called");
     }
 
     size_t bytes_left = len;
     unsigned char const* p = data;
 
     while (bytes_left > 0) {
-        size_t bytes =
-            (bytes_left < this->out_bufsize ? bytes_left : out_bufsize);
+        size_t bytes = (bytes_left < this->out_bufsize ? bytes_left : out_bufsize);
         bytes_left -= bytes;
         // lgtm[cpp/weak-cryptographic-algorithm]
         rc4.process(p, bytes, outbuf.get());
