@@ -1,6 +1,7 @@
 #include <qpdf/QPDFObjectHandle.hh>
 
 #include <qpdf/BufferInputSource.hh>
+#include <qpdf/JSON_writer.hh>
 #include <qpdf/Pl_Buffer.hh>
 #include <qpdf/Pl_QPDFTokenizer.hh>
 #include <qpdf/QPDF.hh>
@@ -1617,10 +1618,33 @@ QPDFObjectHandle::getJSON(int json_version, bool dereference_indirect)
     } else if (!dereference()) {
         throw std::logic_error("attempted to dereference an uninitialized QPDFObjectHandle");
     } else {
-        return obj->getJSON(json_version);
+        Pl_Buffer p{"json"};
+        JSON::Writer jw{&p, 0};
+        writeJSON(json_version, jw, dereference_indirect);
+        p.finish();
+        return JSON::parse(p.getString());
     }
 }
 
+void
+QPDFObjectHandle::writeJSON(int json_version, JSON::Writer& p, bool dereference_indirect)
+{
+    if (!dereference_indirect && isIndirect()) {
+        p << "\"" << getObjGen().unparse(' ') << " R\"";
+    } else if (!dereference()) {
+        throw std::logic_error("attempted to dereference an uninitialized QPDFObjectHandle");
+    } else {
+        obj->writeJSON(json_version, p);
+    }
+}
+
+void
+QPDFObjectHandle::writeJSON(int json_version, Pipeline* p, bool dereference_indirect, size_t depth)
+{
+    JSON::Writer jw{p, depth};
+    writeJSON(json_version, jw, dereference_indirect);
+}
+
 JSON
 QPDFObjectHandle::getStreamJSON(
     int json_version,
