@@ -1,5 +1,7 @@
 #include <qpdf/QPDF_Operator.hh>
 
+#include <qpdf/JSON_writer.hh>
+
 QPDF_Operator::QPDF_Operator(std::string const& val) :
     QPDFValue(::ot_operator, "operator"),
     val(val)
@@ -24,8 +26,8 @@ QPDF_Operator::unparse()
     return val;
 }
 
-JSON
-QPDF_Operator::getJSON(int json_version)
+void
+QPDF_Operator::writeJSON(int json_version, JSON::Writer& p)
 {
-    return JSON::makeNull();
+    p << "null";
 }
