@@ -1,5 +1,6 @@
 #include <qpdf/QPDF_String.hh>
 
+#include <qpdf/JSON_writer.hh>
 #include <qpdf/QUtil.hh>
 
 // DO NOT USE ctype -- it is locale dependent for some things, and it's not worth the risk of
@@ -45,33 +46,27 @@ QPDF_String::unparse()
     return unparse(false);
 }
 
-JSON
-QPDF_String::getJSON(int json_version)
+void
+QPDF_String::writeJSON(int json_version, JSON::Writer& p)
 {
+    auto candidate = getUTF8Val();
     if (json_version == 1) {
-        return JSON::makeString(getUTF8Val());
-    }
-    // See if we can unambiguously represent as Unicode.
-    bool is_unicode = false;
-    std::string result;
-    std::string candidate = getUTF8Val();
-    if (QUtil::is_utf16(this->val) || QUtil::is_explicit_utf8(this->val)) {
-        is_unicode = true;
-        result = candidate;
-    } else if (!useHexString()) {
-        std::string test;
-        if (QUtil::utf8_to_pdf_doc(candidate, test, '?') && (test == this->val)) {
-            // This is a PDF-doc string that can be losslessly encoded as Unicode.
-            is_unicode = true;
-            result = candidate;
-        }
-    }
-    if (is_unicode) {
-        result = "u:" + result;
+        p << "\"" << JSON::Writer::encode_string(candidate) << "\"";
     } else {
-        result = "b:" + QUtil::hex_encode(this->val);
+        // See if we can unambiguously represent as Unicode.
+        if (QUtil::is_utf16(this->val) || QUtil::is_explicit_utf8(this->val)) {
+            p << "\"u:" << JSON::Writer::encode_string(candidate) << "\"";
+            return;
+        } else if (!useHexString()) {
+            std::string test;
+            if (QUtil::utf8_to_pdf_doc(candidate, test, '?') && (test == this->val)) {
+                // This is a PDF-doc string that can be losslessly encoded as Unicode.
+                p << "\"u:" << JSON::Writer::encode_string(candidate) << "\"";
+                return;
+            }
+        }
+        p << "\"b:" << QUtil::hex_encode(val) << "\"";
     }
-    return JSON::makeString(result);
 }
 
 bool
