@@ -9,15 +9,12 @@
 #include <qpdf/QPDFSystemError.hh>
 #include <qpdf/QTC.hh>
 
-#include <cctype>
 #include <cerrno>
-#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <fcntl.h>
 #include <fstream>
 #include <iomanip>
-#include <locale>
 #include <map>
 #include <memory>
 #include <regex>
@@ -1178,21 +1175,43 @@ std::string
 QUtil::read_file_into_string(FILE* f, std::string_view filename)
 {
     fseek(f, 0, SEEK_END);
-    auto size = QIntC::to_size(QUtil::tell(f));
-    fseek(f, 0, SEEK_SET);
-    std::string result(size, '\0');
-    if (auto read = fread(result.data(), 1, size, f); read != size) {
+    auto o_size = QUtil::tell(f);
+    if (o_size >= 0) {
+        // Seekable file
+        auto size = QIntC::to_size(o_size);
+        fseek(f, 0, SEEK_SET);
+        std::string result(size, '\0');
+        if (auto n_read = fread(result.data(), 1, size, f); n_read != size) {
+            if (ferror(f)) {
+                throw std::runtime_error(
+                    std::string("failure reading file ") + std::string(filename) +
+                    " into memory: read " + uint_to_string(n_read) + "; wanted " +
+                    uint_to_string(size));
+            } else {
+                throw std::runtime_error(
+                    std::string("premature eof reading file ") + std::string(filename) +
+                    " into memory: read " + uint_to_string(n_read) + "; wanted " +
+                    uint_to_string(size));
+            }
+        }
+        return result;
+    } else {
+        // Pipe or other non-seekable file
+        size_t buf_size = 8192;
+        auto n_read = buf_size;
+        std::string buffer(buf_size, '\0');
+        std::string result;
+        while (n_read == buf_size) {
+            n_read = fread(buffer.data(), 1, buf_size, f);
+            buffer.erase(n_read);
+            result.append(buffer);
+        }
         if (ferror(f)) {
             throw std::runtime_error(
-                std::string("failure reading file ") + std::string(filename) +
-                " into memory: read " + uint_to_string(read) + "; wanted " + uint_to_string(size));
-        } else {
-            throw std::runtime_error(
-                std::string("premature eof reading file ") + std::string(filename) +
-                " into memory: read " + uint_to_string(read) + "; wanted " + uint_to_string(size));
+                std::string("failure reading file ") + std::string(filename) + " into memory");
         }
+        return result;
     }
-    return result;
 }
 
 static bool
@@ -1281,93 +1300,56 @@ QUtil::str_compare_nocase(char const* s1, char const* s2)
 #endif
 }
 
-static int
-maybe_from_end(int num, bool from_end, int max)
-{
-    if (from_end) {
-        if (num > max) {
-            num = 0;
-        } else {
-            num = max + 1 - num;
-        }
-    }
-    return num;
-}
-
 std::vector<int>
 QUtil::parse_numrange(char const* range, int max)
 {
-    std::vector<int> result;
-    char const* p = range;
+    // Performance note: this implementation aims to be straightforward, not efficient. Numeric
+    // range parsing is used only during argument processing. It is not used during processing of
+    // PDF files.
+
+    static std::regex group_re(R"((x)?(z|r?\d+)(?:-(z|r?\d+))?)");
+    auto parse_num = [&max](std::string const& s) -> int {
+        if (s == "z") {
+            return max;
+        }
+        int num;
+        if (s.at(0) == 'r') {
+            num = max + 1 - string_to_int(s.substr(1).c_str());
+        } else {
+            num = string_to_int(s.c_str());
+        }
+        // max == 0 means we don't know the max and are just testing for valid syntax.
+        if ((max > 0) && ((num < 1) || (num > max))) {
+            throw std::runtime_error("number " + std::to_string(num) + " out of range");
+        }
+        return num;
+    };
+
+    auto populate = [](std::vector<int>& group, int first_num, bool is_span, int last_num) {
+        group.clear();
+        group.emplace_back(first_num);
+        if (is_span) {
+            if (first_num > last_num) {
+                for (auto i = first_num - 1; i >= last_num; --i) {
+                    group.push_back(i);
+                }
+            } else {
+                for (auto i = first_num + 1; i <= last_num; ++i) {
+                    group.push_back(i);
+                }
+            }
+        }
+    };
+
+    char const* p;
     try {
-        std::vector<int> work;
-        static int const comma = -1;
-        static int const dash = -2;
+        char const* range_end = range + strlen(range);
+        std::vector<int> result;
+        std::vector<int> last_group;
+        // See if range ends with :even or :odd.
         size_t start_idx = 0;
         size_t skip = 1;
-
-        enum { st_top, st_in_number, st_after_number } state = st_top;
-        bool last_separator_was_dash = false;
-        int cur_number = 0;
-        bool from_end = false;
-        while (*p) {
-            char ch = *p;
-            if (isdigit(ch)) {
-                if (!((state == st_top) || (state == st_in_number))) {
-                    throw std::runtime_error("digit not expected");
-                }
-                state = st_in_number;
-                cur_number *= 10;
-                cur_number += (ch - '0');
-            } else if (ch == 'z') {
-                // z represents max
-                if (!(state == st_top)) {
-                    throw std::runtime_error("z not expected");
-                }
-                state = st_after_number;
-                cur_number = max;
-            } else if (ch == 'r') {
-                if (!(state == st_top)) {
-                    throw std::runtime_error("r not expected");
-                }
-                state = st_in_number;
-                from_end = true;
-            } else if ((ch == ',') || (ch == '-')) {
-                if (!((state == st_in_number) || (state == st_after_number))) {
-                    throw std::runtime_error("unexpected separator");
-                }
-                cur_number = maybe_from_end(cur_number, from_end, max);
-                work.push_back(cur_number);
-                cur_number = 0;
-                from_end = false;
-                if (ch == ',') {
-                    state = st_top;
-                    last_separator_was_dash = false;
-                    work.push_back(comma);
-                } else if (ch == '-') {
-                    if (last_separator_was_dash) {
-                        throw std::runtime_error("unexpected dash");
-                    }
-                    state = st_top;
-                    last_separator_was_dash = true;
-                    work.push_back(dash);
-                }
-            } else if (ch == ':') {
-                if (!((state == st_in_number) || (state == st_after_number))) {
-                    throw std::runtime_error("unexpected colon");
-                }
-                break;
-            } else {
-                throw std::runtime_error("unexpected character");
-            }
-            ++p;
-        }
-        if ((state == st_in_number) || (state == st_after_number)) {
-            cur_number = maybe_from_end(cur_number, from_end, max);
-            work.push_back(cur_number);
-        } else {
-            throw std::runtime_error("number expected");
-        }
+        p = std::find(range, range_end, ':');
         if (*p == ':') {
             if (strcmp(p, ":odd") == 0) {
                 skip = 2;
@@ -1375,46 +1357,65 @@ QUtil::parse_numrange(char const* range, int max)
                 skip = 2;
                 start_idx = 1;
             } else {
-                throw std::runtime_error("unexpected even/odd modifier");
+                throw std::runtime_error("expected :even or :odd");
             }
+            range_end = p;
         }
 
-        p = nullptr;
-        for (size_t i = 0; i < work.size(); i += 2) {
-            int num = work.at(i);
-            // max == 0 means we don't know the max and are just testing for valid syntax.
-            if ((max > 0) && ((num < 1) || (num > max))) {
-                throw std::runtime_error("number " + QUtil::int_to_string(num) + " out of range");
+        // Divide the range into groups
+        p = range;
+        char const* group_end;
+        bool first = true;
+        while (p != range_end) {
+            group_end = std::find(p, range_end, ',');
+            std::cmatch m;
+            if (!std::regex_match(p, group_end, m, group_re)) {
+                throw std::runtime_error("invalid range syntax");
             }
-            if (i == 0) {
-                result.push_back(work.at(i));
-            } else {
-                int separator = work.at(i - 1);
-                if (separator == comma) {
-                    result.push_back(num);
-                } else if (separator == dash) {
-                    int lastnum = result.back();
-                    if (num > lastnum) {
-                        for (int j = lastnum + 1; j <= num; ++j) {
-                            result.push_back(j);
-                        }
-                    } else {
-                        for (int j = lastnum - 1; j >= num; --j) {
-                            result.push_back(j);
-                        }
+            auto is_exclude = m[1].matched;
+            if (first && is_exclude) {
+                throw std::runtime_error("first range group may not be an exclusion");
+            }
+            first = false;
+            auto first_num = parse_num(m[2].str());
+            auto is_span = m[3].matched;
+            int last_num{0};
+            if (is_span) {
+                last_num = parse_num(m[3].str());
+            }
+            if (is_exclude) {
+                std::vector<int> work;
+                populate(work, first_num, is_span, last_num);
+                std::set<int> exclusions;
+                exclusions.insert(work.begin(), work.end());
+                work = last_group;
+                last_group.clear();
+                for (auto n: work) {
+                    if (exclusions.count(n) == 0) {
+                        last_group.emplace_back(n);
                     }
-                } else {
-                    throw std::logic_error("INTERNAL ERROR parsing numeric range");
+                }
+            } else {
+                result.insert(result.end(), last_group.begin(), last_group.end());
+                populate(last_group, first_num, is_span, last_num);
+            }
+            p = group_end;
+            if (*p == ',') {
+                ++p;
+                if (p == range_end) {
+                    throw std::runtime_error("trailing comma");
                 }
             }
         }
-        if ((start_idx > 0) || (skip != 1)) {
-            auto t = result;
-            result.clear();
-            for (size_t i = start_idx; i < t.size(); i += skip) {
-                result.push_back(t.at(i));
-            }
+        result.insert(result.end(), last_group.begin(), last_group.end());
+        if (skip == 1) {
+            return result;
         }
+        std::vector<int> filtered;
+        for (auto i = start_idx; i < result.size(); i += skip) {
+            filtered.emplace_back(result.at(i));
+        }
+        return filtered;
     } catch (std::runtime_error const& e) {
         std::string message;
         if (p) {
@@ -1425,7 +1426,6 @@ QUtil::parse_numrange(char const* range, int max)
         }
         throw std::runtime_error(message);
     }
-    return result;
 }
 
 enum encoding_e { e_utf16, e_ascii, e_winansi, e_macroman, e_pdfdoc };
@@ -1463,6 +1463,7 @@ encode_pdfdoc(unsigned long codepoint)
 unsigned long
 QUtil::get_next_utf8_codepoint(std::string const& utf8_val, size_t& pos, bool& error)
 {
+    auto o_pos = pos;
     size_t len = utf8_val.length();
     unsigned char ch = static_cast<unsigned char>(utf8_val.at(pos++));
     error = false;
@@ -1483,7 +1484,7 @@ QUtil::get_next_utf8_codepoint(std::string const& utf8_val, size_t& pos, bool& e
         return 0xfffd;
     }
 
-    unsigned long codepoint = static_cast<unsigned long>(ch & ~to_clear);
+    auto codepoint = static_cast<unsigned long>(ch & ~to_clear);
     while (bytes_needed > 0) {
         --bytes_needed;
         ch = static_cast<unsigned char>(utf8_val.at(pos++));
@@ -1495,6 +1496,31 @@ QUtil::get_next_utf8_codepoint(std::string const& utf8_val, size_t& pos, bool& e
         codepoint <<= 6;
         codepoint += (ch & 0x3f);
     }
+    unsigned long lower_bound = 0;
+    switch (pos - o_pos) {
+    case 2:
+        lower_bound = 1 << 7;
+        break;
+    case 3:
+        lower_bound = 1 << 11;
+        break;
+    case 4:
+        lower_bound = 1 << 16;
+        break;
+    case 5:
+        lower_bound = 1 << 12;
+        break;
+    case 6:
+        lower_bound = 1 << 26;
+        break;
+    default:
+        lower_bound = 0;
+    }
+
+    if (lower_bound > 0 && codepoint < lower_bound) {
+        // Too many bytes were used, but return whatever character was encoded.
+        error = true;
+    }
     return codepoint;
 }
 
@@ -1777,11 +1803,12 @@ QUtil::analyze_encoding(
     bool any_errors = false;
     while (pos < len) {
         bool error = false;
-        unsigned long codepoint = get_next_utf8_codepoint(val, pos, error);
+        auto o_pos = pos;
+        get_next_utf8_codepoint(val, pos, error);
         if (error) {
             any_errors = true;
         }
-        if (codepoint >= 128) {
+        if (pos - o_pos > 1 || val[o_pos] & 0x80) {
             has_8bit_chars = true;
         }
     }
